// server.js

var session = require('connect').session
  , cookieParser = require('connect').cookieParser
  , flatiron = require('flatiron')
  , app = flatiron.app
  , fs = require('fs')
  , resourceful = require('resourceful')
  , qs = require('querystring')
  , plates = require('plates')
  ;

var Task = resourceful.define('task', function() {
  this.timestamps();
  this.string('desc').required(true)
                     .minLength(1);
});

app.use(flatiron.plugins.http);

app.http.before.push(cookieParser('todo list secret'));
app.http.before.push(session({cookie: {maxAge: 60000}}));
app.http.before.push(function(req, res, next) {
  console.log('------------------------------------------------------------');
  console.log(req);
  console.log('------------------------------------------------------------');
  console.log(res);
  console.log('------------------------------------------------------------');
  console.log(next);
  console.log('------------------------------------------------------------');
  next();
});

app.router.get('/', function() {
  var self = this;
  fs.readFile('index.html', function(err, data) {
    if(err) {
      self.res.writeHead(404);
      self.res.end();
      return;
    }
    var listContent = "";
    var listItemTemplate = '<button>Delete Item</button>' +
                           '<span id="ctime" class="ctime"></span>' +
                           'I need to <span id="desc" class="desc"></span>' +
                           '<input type="hidden" name="method" value="DELETE" />' +
                           '<input type="hidden" id="id" name="id" value="" />';
    if(typeof self.req.session.tasks === 'undefined') {
      console.log("No tasks in session.");
      self.req.session.tasks = [];
    }
    var options = {
      'id': ['id', 'value'],
      'ctime': 'id',
      'desc': 'id'
    };
    self.req.session.tasks.forEach(function(task) {
      var data = {
        'desc': task.desc,
        'ctime': new Date(task.ctime).toDateString().substr(4),
        'id': task._id
      };
      listContent += '<li>' +
        '<form method="post" action="/">' +
        plates.bind(listItemTemplate, data, options) +
        '</form>' +
      '</li>';
    });
    data = plates.bind(data, {'task-list': listContent});
    self.res.writeHead(200, {'Content-Type': 'text/html'});
    self.res.end(data);
  })
});

app.router.post('/', function() {
  var self = this;
  if(self.req.body.method && self.req.body.method === "DELETE") {
    var taskId = self.req.body.id;
    var task = Task.get(taskId);
    var tasks = self.req.session.tasks;
    tasks = tasks.splice(0, tasks.indexOf(task)).concat(tasks.splice(1));
    self.req.session.tasks = tasks;
    Task.destroy(taskId);
  } else {
    var task = new (Task)({desc: self.req.body.desc});
    if(task.validate().valid) {
      task.save();
      if(!self.req.session.tasks) {
        self.req.session.tasks = [];
      }
      self.req.connection = {encrypted: false};
      self.res.emit('header');
      self.req.session.tasks.push(task);
    }
  }
  self.res.writeHead(303, {'Location': '/'});
  self.res.end();
});

app.start(8090);
